package info.aspirecn.iov.sjjh.supplier10000053.util;

import com.alibaba.fastjson.JSONObject;
import info.aspirecn.iov.sjjh.commons.lang.Constant;
import lombok.extern.slf4j.Slf4j;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.springframework.util.Base64Utils;
import sun.misc.BASE64Decoder;
import sun.misc.BASE64Encoder;

import javax.crypto.Cipher;
import javax.crypto.SecretKey;
import javax.crypto.SecretKeyFactory;
import javax.crypto.spec.DESKeySpec;
import javax.crypto.spec.DESedeKeySpec;
import javax.crypto.spec.IvParameterSpec;
import java.security.Key;
import java.security.Security;
import java.security.spec.AlgorithmParameterSpec;
import java.util.HashMap;
import java.util.Map;

/**
 * 3DES算法。
 * java的api中仅仅提供了DES,DESede和PBE 3三种对称加密算法密钥材料实现类
 * <p>
 * 使用CBC需要增加加密向量
 * - 第一段是加密算法的名称，如DESede实际上是3-DES。这一段还可以放其它的对称加密算法，如Blowfish等。
 * - 第二段是分组加密的模式，除了CBC和ECB之外，还可以是NONE/CFB/QFB等。最常用的就是CBC和ECB了。DES采用分组加密的方式，将明文按8字节（64位）分组分别加密。如果每个组独立处理，则是ECB。CBC的处理方式是先用初始向量IV对第一组加密，再用第一组的密文作为密钥对第二组加密，然后依次完成整个加密操作。如果明文中有两个分组的内容相同，ECB会得到完全一样的密文，但CBC则不会。
 * - 第三段是指最后一个分组的填充方式。大部分情况下，明文并非刚好64位的倍数。对于最后一个分组，如果长度小于64位，则需要用数据填充至64位。PKCS5Padding是常用的填充方式，如果没有指定，默认的方式就是它。
 * 补充一点，虽然DES的有效密钥长度是56位，但要求密钥长度是64位（8字节）。3DES则要求24字节。
 * <p>
 * <p>
 * CBC使用加密向量
 * ECB不使用加密向量
 * <p>
 * 2020-03-07
 */
@Slf4j
public class Des3Util {

    public static final String ALGORITHM = "DESede";
    public static final String PKCS5 = "pkcs5";
    public static final String PKCS7 = "pkcs7";
    public static final String CBC = "CBC";
    public static final String DESEDE = "DESede/";

    private Des3Util() {

    }

    static {
        if (Security.getProvider(BouncyCastleProvider.PROVIDER_NAME) == null) {

            log.info("security provider BC not found");

            Security.addProvider(new BouncyCastleProvider());

        }
    }

    /**
     * 加密
     *
     * @return
     * @throws Exception
     */
    public static String encrypt(String secretMsg, String secretKey, String secretModel, String secretVector, String secretEncryptionMode) throws Exception {
        String encryptMsg = null;
        if (PKCS5.equalsIgnoreCase(secretModel)) {
            //加密
            //加解密信息，秘钥，加密向量，加密模式(CBC或者ECB)
            encryptMsg = encryptDESCBC(secretMsg, secretKey, secretVector, secretEncryptionMode);
        }
        if (PKCS7.equalsIgnoreCase(secretModel)) {
            //加密
            //加解密信息，秘钥,加密向量.加密模式(CBC或者ECB)
            encryptMsg = encryptThreeDESECB(secretMsg, secretKey, secretVector, secretEncryptionMode);
        }
        return encryptMsg;
    }


    /**
     * 解密
     *
     * @return
     * @throws Exception
     */
    public static String decrypt(String secretMsg, String secretKey, String secretModel, String secretVector, String secretEncryptionMode) throws Exception {
        String decryptMsg = null;
        if (PKCS5.equalsIgnoreCase(secretModel)) {

            //解密
            //加解密信息，秘钥，加密向量
            decryptMsg = decryptDESCBC(secretMsg, secretKey, secretVector, secretEncryptionMode);
        }
        if (PKCS7.equalsIgnoreCase(secretModel)) {
            //解密
            //加解密信息，秘钥
            decryptMsg = decryptThreeDESECB(secretMsg, secretKey, secretVector, secretEncryptionMode);
        }
        return decryptMsg;
    }


    /**
     * DESCBC加密
     *
     * @param src              加密信息
     * @param key              秘钥
     * @param encryptionVector 加密向量
     * @param algorithm        CBC或者ECB
     * @return 返回加密后的数据
     * @throws Exception
     */
    public static String encryptDESCBC(final String src, final String key, final String encryptionVector, final String algorithm) throws Exception {
        SecretKey sKsecretKey = getKey(key);

        final Cipher cipher = Cipher.getInstance(DESEDE + algorithm + "/PKCS5Padding");

        if (CBC.equalsIgnoreCase(algorithm)) {
            // --加密向量
            final IvParameterSpec iv = new IvParameterSpec(encryptionVector.getBytes(Constant.PARA_ENCODE_UTF8));
            cipher.init(Cipher.ENCRYPT_MODE, sKsecretKey, iv);
        } else {
            cipher.init(Cipher.ENCRYPT_MODE, sKsecretKey);
        }

        final byte[] b = cipher.doFinal(src.getBytes(Constant.PARA_ENCODE_UTF8));

        // --通过base64,将加密数组转换成字符串
        final BASE64Encoder encoder = new BASE64Encoder();
        return encoder.encode(b);
    }

    /**
     * DESCBC解密
     *
     * @param src              密文信息
     * @param key              秘钥
     * @param encryptionVector 加密向量
     * @param algorithm        CBC或者ECB
     * @return 返回解密后的原始数据
     * @throws Exception
     */
    public static String decryptDESCBC(final String src, final String key, final String encryptionVector, final String algorithm) throws Exception {
        // --通过base64,将字符串转成byte数组
        final BASE64Decoder decoder = new BASE64Decoder();
        final byte[] bytesrc = decoder.decodeBuffer(src);

        SecretKey sKsecretKey = getKey(key);

        final Cipher cipher = Cipher.getInstance(DESEDE + algorithm + "/PKCS5Padding");

        if (CBC.equalsIgnoreCase(algorithm)) {
            // --加密向量
            final IvParameterSpec iv = new IvParameterSpec(encryptionVector.getBytes(Constant.PARA_ENCODE_UTF8));
            cipher.init(Cipher.DECRYPT_MODE, sKsecretKey, iv);
        } else {
            cipher.init(Cipher.DECRYPT_MODE, sKsecretKey);
        }

        final byte[] retByte = cipher.doFinal(bytesrc);

        return new String(retByte);

    }

    /**
     * @param src              密文信息
     * @param key              秘钥
     * @param encryptionVector 加密向量
     * @param algorithm        CBC或者ECB
     * @return
     * @throws Exception
     */
    public static String encryptThreeDESECB(final String src, final String key, final String encryptionVector, final String algorithm) throws Exception {
        /**
         * 这个地方调用BouncyCastleProvider 让java支持PKCS7Padding
         */
        /*Security.addProvider(new BouncyCastleProvider());*/


        SecretKey sKsecretKey = getKey(key);

        final Cipher cipher = Cipher.getInstance(DESEDE + algorithm + "/PKCS7Padding");

        if (CBC.equalsIgnoreCase(algorithm)) {
            final IvParameterSpec iv = new IvParameterSpec(encryptionVector.getBytes(Constant.PARA_ENCODE_UTF8));
            cipher.init(Cipher.ENCRYPT_MODE, sKsecretKey, iv);
        } else {
            cipher.init(Cipher.ENCRYPT_MODE, sKsecretKey);
        }

        final byte[] b = cipher.doFinal(src.getBytes());

        final BASE64Encoder encoder = new BASE64Encoder();
        return encoder.encode(b).replaceAll("\r", "").replaceAll("\n", "");

    }

    // 3DESECB解密,key必须是长度大于等于 3*8 = 24 位哈 //加解密信息，秘钥,加密向量.加密模式(CBC或者ECB)
    public static String decryptThreeDESECB(final String src, final String key, final String encryptionVector, final String algorithm) throws Exception {
        /**
         * 这个地方调用BouncyCastleProvider 让java支持PKCS7Padding
         */
        /*Security.addProvider(new BouncyCastleProvider());*/
        // --通过base64,将字符串转成byte数组
        final BASE64Decoder decoder = new BASE64Decoder();
        final byte[] bytesrc = decoder.decodeBuffer(src);
        // --解密的key
        SecretKey sKsecretKey = getKey(key);

        // --Chipher对象解密
        final Cipher cipher = Cipher.getInstance(DESEDE + algorithm + "/PKCS7Padding");

        if (CBC.equalsIgnoreCase(algorithm)) {
            final IvParameterSpec iv = new IvParameterSpec(encryptionVector.getBytes(Constant.PARA_ENCODE_UTF8));
            cipher.init(Cipher.DECRYPT_MODE, sKsecretKey, iv);
        } else {
            cipher.init(Cipher.DECRYPT_MODE, sKsecretKey);
        }

        final byte[] retByte = cipher.doFinal(bytesrc);

        return new String(retByte);
    }

    /**
     * 使用DESede对secretKey进行加密
     *
     * @param secretKey 密钥
     * @throws Exception
     */
    public static SecretKey getKey(String secretKey) throws Exception {
        byte[] keyBytes = new BASE64Decoder().decodeBuffer(secretKey);
        DESedeKeySpec dks = new DESedeKeySpec(keyBytes);
        SecretKeyFactory keyFactory = SecretKeyFactory.getInstance(ALGORITHM);
        SecretKey sKsecretKey = keyFactory.generateSecret(dks);
        return sKsecretKey;
    }

    /**
     * 加密
     *
     * @param key
     * @param data
     * @return
     * @throws Exception
     */
    public static String encode(String key, String data) {

        try {
            DESKeySpec dks = new DESKeySpec(key.getBytes());
            SecretKeyFactory keyFactory = SecretKeyFactory.getInstance("DES");
            // key的长度不能够小于8位字节
            Key secretKey = keyFactory.generateSecret(dks);
            Cipher cipher = Cipher.getInstance("DES/CBC/PKCS5Padding");
            IvParameterSpec iv = new IvParameterSpec(key.getBytes());// 向量
            AlgorithmParameterSpec paramSpec = iv;
            cipher.init(Cipher.ENCRYPT_MODE, secretKey, paramSpec);
            byte[] bytes = cipher.doFinal(data.getBytes("utf-8"));
            return Base64Utils.encodeToString(bytes);
        } catch (Exception e) {
            e.printStackTrace();
            return "";
        }
    }

    /**
     * 解密
     *
     * @param key
     * @return
     * @throws Exception
     */
    public static byte[] decode(String key, byte[] data) throws Exception {
        try {
            DESKeySpec dks = new DESKeySpec(key.getBytes());
            SecretKeyFactory keyFactory = SecretKeyFactory.getInstance("DES");
            // key的长度不能够小于8位字节
            Key secretKey = keyFactory.generateSecret(dks);
            Cipher cipher = Cipher.getInstance("DES/CBC/PKCS5Padding");
            IvParameterSpec iv = new IvParameterSpec(key.getBytes());
            AlgorithmParameterSpec paramSpec = iv;
            cipher.init(Cipher.DECRYPT_MODE, secretKey, paramSpec);
            byte[] bs = cipher.doFinal(data);
            return bs;
        } catch (Exception e) {
            log.error("e:{}", e);
            throw e;
        }
    }

    public static void main(String[] args) throws Exception {
        String str = "{\"empowerNo\":\"21082404253016297935\",\"mobile\":\"18842830710\"}";

        Map<String, String> request = new HashMap<>();
        request.put("mobile", "18842830710");
        //request.put("certNo","41088119930617752X");
        request.put("empowerNo", "21082403155088337101");
        //request.put("date","20210507");
        String src = JSONObject.toJSONString(request);
        log.info("src:{}", src);
        String key = "123@abc!";
        String result = Des3Util.encode(key, src);
        System.out.println("result:" + result);
        System.out.println(new String(Des3Util.decode(key, Base64Utils.decodeFromString("8v0pqQ2UaWXP+ZGojtdH4OVsEGnc6TPDPy467nGJbf8iUwVU4RzT7/8phVVg6ScKz5Qma+NJ87GvtLoxoMEmlg=="))));
        System.out.println(Des3Util.encode(key, str));
    }


    private static String encode = "UTF-8";


}
